<?php

/**
 * Model to handle inline mod-style actions on threads. Generally, these are simply
 * bulk actions. They can be applied to other circumstances if desired.
 *
 * @package XenForo_Thread
 */
class SK_MarkThreadRead_Model_InlineMod_Thread extends XFCP_SK_MarkThreadRead_Model_InlineMod_Thread
{
	/**
	 * Marks the specified threads as read.
	 *
	 * @param array $threadIds List of thread IDs to undelete
	 * @param array $options Options that control the action. Nothing supported at this time.
	 * @param string $errorKey Modified by reference. If no permission, may include a key of a phrase that gives more info
	 * @param array|null $viewingUser
	 *
	 * @return boolean True if permissions were ok
	 */
	public function markThreadsRead(array $threadIds, array $options = array(), &$errorKey = '', array $viewingUser = null)
	{
		list($threads, $forums) = $this->getThreadsAndParentData($threadIds, $viewingUser);
		
		$this->standardizeViewingUserReference($viewingUser);
		
		$readDates = $this->_getUserThreadReadDates($viewingUser['user_id'], $threadIds);
	
		$autoReadDate = XenForo_Application::$time - (XenForo_Application::get('options')->readMarkingDataLifetime * 86400);
		$threadModel = $this->_getThreadModel();
		
		foreach ($threads AS $thread)
		{
			$thread['thread_read_date'] =
				(isset($readDates[$thread['thread_id']]) ?
					max($readDates[$thread['thread_id']]['thread_read_date'], $autoReadDate) :
					$autoReadDate); 
		
			$forum = $this->_getForumFromThread($thread, $forums);
			
			$threadModel->markThreadRead($thread, $forum, XenForo_Application::$time, $viewingUser);
		}
		
		return true;
	}
	
	/**
	 * Get the time when a user has marked the given thread as read.
	 *
	 * @param integer $userId
	 * @param integer $threadIds
	 *
	 * @return array
	 */
	protected function _getUserThreadReadDates($userId, $threadIds)
	{
		if (!$userId)
		{
			return array();
		}
		
		return $this->fetchAllKeyed('
			SELECT thread_id, thread_read_date
			FROM xf_thread_read
			WHERE user_id = ? AND thread_id IN (' . $this->_getDb()->quote($threadIds) . ')
		', 'thread_id', array($userId));
	}
	
}